module BaselineUtil
  unloadable
  def self.day_ending(date)
    SETTINGS.baseline_day_end ||= "2400"
    date.to_time + ( (SETTINGS.baseline_day_end.to_i / 100).hours + (SETTINGS.baseline_day_end.to_i % 100).minutes)
  end

  def self.day_beginning(date)
    day_ending(date) - 24.hours
  end


  def self.create_missing_baselines_for_node(node)
    unless node.last_inspect_report_id
      raise "Can't create missing baselines for node #{node.name} without an inspect report"
    end

    nb = NodeBaseline.self_comparisons.
      find_by_node_id(node.id, :order => 'date DESC')

    unless nb
      report = node.last_inspect_report
      nb = NodeBaseline.
        create_from_report!(report, BaselineUtil.day_ending(report.time))
    end

    # Create the right number of baselines by walking through and
    # creating the next in the series; nb is updated to the next
    # baseline each round.
    (Date.today - nb.date).to_i.times do
      nb = nb.create_for_next_day
    end
  end

  def self.other_day_summaries(date, node_baseline_ids=nil)
    date = date.to_s(:db)

    node_baselines = node_baseline_ids ?
      NodeBaseline.order(:date).scoped_by_id(node_baseline_ids) :
      NodeBaseline.order(:date)

    return [] unless node_baselines.present?

    first_day = node_baselines.first.date
    last_day  = node_baselines.last.date
    all_days = (first_day..last_day)

    day_choices = all_days.inject({}) do |choices, day|
      choices.merge(day.to_s(:db) => day.to_s(:db))
    end

    diffs = BaselineDifference
    diffs = diffs.scoped_by_node_baseline_id(node_baseline_ids) if node_baseline_ids

    day_counts = diffs.
      undecided.
      self_comparisons.
      for_dates_other_than(date).all(
      :group  => '`node_baselines`.`date`',
      :select => ['`node_baselines`.`date`, count(*) as unreviewed_count']
    )

    day_counts.each do |day_count|
      label = day_count.date
      if day_count.unreviewed_count.to_i > 0
        label += " (#{day_count.unreviewed_count} unreviewed)"
      end

      day_choices[day_count.date] = label
    end

    # This is a date changer, not a date stay-the-samer.
    day_choices.delete(date)

    day_choices.map do |day, label|
      [label, day]
    end.sort { |a,b| b.last <=> a.last }
  end

  def self.summary_for_day(date, node_ids)
    baselines = NodeBaseline.self_comparisons.scoped_by_node_id_and_date(node_ids, date)

    if baselines.present?
      {
        :unreviewed_nodes => baselines.with_diffs_by_status(nil).count(:node_id, :distinct => true),
        :unreviewed       => baselines.with_diffs_by_status(nil).count,
        :accepted         => baselines.with_diffs_by_status(true).count,
        :rejected         => baselines.with_diffs_by_status(false).count
      }
    else
      {
        :unreviewed_nodes => 'N/A',
        :unreviewed       => 'N/A',
        :accepted         => 'N/A',
        :rejected         => 'N/A',
      }
    end
  end

  def self.unreviewed_changes_on_other_days(date, node_ids)
    unreviewed_changes_on_other_days = BaselineDifference.
      self_comparisons.
      for_dates_other_than(date).
      scoped_by_node_id(node_ids).
      undecided.
      count
  end

  def self.date_changer_options(date, node_ids)
    baselines = NodeBaseline.self_comparisons.scoped_by_node_id(node_ids)

    BaselineUtil.other_day_summaries(date, baselines.map(&:id))
  end
end
